﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace EPT_Analog_Monitor
{
    public partial class EPT_Analog_Monitor : Form
    {
        public EPT_Analog_Monitor()
        {
            InitializeComponent();

            for (int i = 0; i < EPTTransmitDevice.Length; ++i)
            {
                EPTTransmitDevice[i] = new Transfer();
            }
        }

        //Index to store device selection
        Int32 device_index;

        //Create a Receive object of the Transfer Class.
        Transfer EPTReceiveData = new Transfer();

        //Create an array of the Transfer Class for device
        Transfer[] EPTTransmitDevice = new Transfer[8];

        //Parameters for EPT Receive object
        public const byte TRIGGER_OUT_COMMAND = 0x1;
        public const byte TRANSFER_OUT_COMMAND = 0x2;
        public const byte BLOCK_OUT_COMMAND = 0x4;

        //Parameters for EPT Receive object
        public const byte TRANSFER_OUT_ADDRESS_1 = 0x1;
        public const byte TRANSFER_OUT_ADDRESS_2 = 0x2;
        public const byte TRANSFER_OUT_ADDRESS_3 = 0x3;
        public const byte TRANSFER_OUT_ADDRESS_4 = 0x4;
        public const byte TRANSFER_OUT_ADDRESS_5 = 0x5;
        public const byte TRANSFER_OUT_ADDRESS_6 = 0x6;

        //Address variables to keep track of which Transfer
        //group should store the Display byte from the EPT-570
        public bool DisplayAddress_1 = false;
        public bool DisplayAddress_2 = false;
        public bool DisplayAddress_3 = false;
        public bool DisplayAddress_4 = false;
        public bool DisplayAddress_5 = false;
        public bool DisplayAddress_6 = false;

        //Register to keep track of which byte (Upper or Lower)
        //should be assembled into the AnalogValue
        public bool FirstDisplayByte = true;

        //Address variables to store history of the addresses
        //from the EPT Receive object
        public int FirstPreviousActualAddr = 0;
        public int SecondPreviousActualAddr = 0;

        //Address variables to store history of the addresses
        //determined by the case statements 
        public int FirstPreviousCalcAddr = 0;
        public int SecondPreviousCalcAddr = 0;

        //Variable to store the 10 bit Analog Value
        public int AnalogValue = 0;
        public float FloatValue = 0;

        // Main object loader
        private void EPT_Analog_Monitor_Load(object sender, System.EventArgs e)
        {

            // Call the List Devices function
            ListDevices();
        }

        private void SetButtonEnables()
        {
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;
        }

        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            //Open the Device
            OpenDevice();
            lblDeviceConnected.Text = "Device Connected";
        }

        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            EPT_AH_CloseDeviceByIndex(device_index);
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            lblDeviceConnected.Text = " ";
        }


        private void btnOk_Click(object sender, EventArgs e)
        {
            EPT_AH_CloseDeviceByIndex(device_index);
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            EPT_AH_CloseDeviceByIndex(device_index);
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            lblDeviceConnected.Text = "";
            Application.Exit();
        }

        private void EPTParseReceive()
        {
            switch (EPTReceiveData.Command)
            {
                case TRANSFER_OUT_COMMAND:
                    TransferOutReceive();
                    break;
                default:
                    break;
            }
        }


        public void TransferOutReceive()
        {

            //Store the address history from the EPT Receive Object
            SecondPreviousActualAddr = FirstPreviousActualAddr;
            FirstPreviousActualAddr = EPTReceiveData.Address;
            switch (EPTReceiveData.Address)
            {
                case TRANSFER_OUT_ADDRESS_1:
                    // Send the one character buffer.
                    if (DisplayAddress_1 & FirstDisplayByte)
                    {
                        AnalogValue = 0;
                        FirstDisplayByte = false;
                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                    }
                    else if (DisplayAddress_1 & !FirstDisplayByte)
                    {
                        string WriteRcvChar = "";
                        FirstDisplayByte = true;

                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                        if (ScaleFactor1 != 0.0)
                        {
                            FloatValue = AnalogValue * ScaleFactor1;
                            WriteRcvChar = String.Format("{0}", FloatValue);
                        }
                        else
                            WriteRcvChar = String.Format("{0}", AnalogValue);

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = true;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                        
                        Thread Display1Thread = new Thread(new ParameterizedThreadStart(DisplayValue1));
                        Display1Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_1 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = true;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                    }
                    else if (!DisplayAddress_1 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = true;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                        else
                        {
                            DisplayAddress_1 = true;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                    }
                    break;
                case TRANSFER_OUT_ADDRESS_2:
                    // Send the one character buffer.
                    if (DisplayAddress_2 & FirstDisplayByte)
                    {
                        AnalogValue = 0;
                        FirstDisplayByte = false;
                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                    }
                    else if (DisplayAddress_2 & !FirstDisplayByte)
                    {
                        string WriteRcvChar = "";
                        FirstDisplayByte = true;

                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                        if (ScaleFactor2 != 0)
                        {
                            FloatValue = AnalogValue * ScaleFactor2;
                            WriteRcvChar = String.Format("{0}", FloatValue);
                        }
                        else
                            WriteRcvChar = String.Format("{0}", AnalogValue);

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = true;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                        
                    Thread Display2Thread = new Thread(new ParameterizedThreadStart(DisplayValue2));
                    Display2Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_2 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = true;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                    }
                    else if (!DisplayAddress_2 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = true;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                        else
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = true;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                    }
                   break;
                case TRANSFER_OUT_ADDRESS_3:
                    // Send the one character buffer.
                    if (DisplayAddress_3 & FirstDisplayByte)
                    {
                        AnalogValue = 0;
                        FirstDisplayByte = false;
                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                    }
                    else if (DisplayAddress_3 & !FirstDisplayByte)
                    {
                        string WriteRcvChar = "";
                        FirstDisplayByte = true;

                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                        if (ScaleFactor3 != 0)
                        {
                            FloatValue = AnalogValue * ScaleFactor3;
                            WriteRcvChar = String.Format("{0}", FloatValue);
                        }
                        else
                            WriteRcvChar = String.Format("{0}", AnalogValue);

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = true;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                        
                    Thread Display3Thread = new Thread(new ParameterizedThreadStart(DisplayValue3));
                    Display3Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_3 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = true;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                    }
                    else if (!DisplayAddress_3 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = true;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                        else
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = true;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                    }
                   break;
                case TRANSFER_OUT_ADDRESS_4:
                    // Send the one character buffer.
                    if (DisplayAddress_4 & FirstDisplayByte)
                    {
                        AnalogValue = 0;
                        FirstDisplayByte = false;
                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                    }
                    else if (DisplayAddress_4 & !FirstDisplayByte)
                    {
                        string WriteRcvChar = "";
                        FirstDisplayByte = true;

                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                        if (ScaleFactor4 != 0)
                        {
                            FloatValue = AnalogValue * ScaleFactor4;
                            WriteRcvChar = String.Format("{0}", FloatValue);
                        }
                        else
                            WriteRcvChar = String.Format("{0}", AnalogValue);

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = true;
                        DisplayAddress_6 = false;
                        
                    Thread Display4Thread = new Thread(new ParameterizedThreadStart(DisplayValue4));
                    Display4Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_4 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = true;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = false;
                    }
                    else if (!DisplayAddress_4 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = true;
                            DisplayAddress_6 = false;
                        }
                        else
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = true;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                    }
                   break;
                case TRANSFER_OUT_ADDRESS_5:
                    // Send the one character buffer.
                    if (DisplayAddress_5 & FirstDisplayByte)
                    {
                        AnalogValue = 0;
                        FirstDisplayByte = false;
                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                    }
                    else if (DisplayAddress_5 & !FirstDisplayByte)
                    {
                        string WriteRcvChar = "";
                        FirstDisplayByte = true;

                        AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                        if (ScaleFactor5 != 0)
                        {
                            FloatValue = AnalogValue * ScaleFactor5;
                            WriteRcvChar = String.Format("{0}", FloatValue);
                        }
                        else
                            WriteRcvChar = String.Format("{0}", AnalogValue);

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = true;
                        
                    Thread Display5Thread = new Thread(new ParameterizedThreadStart(DisplayValue5));
                    Display5Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_5 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = true;
                        DisplayAddress_6 = false;
                    }
                    else if (!DisplayAddress_5 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = true;
                        }
                        else
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = true;
                            DisplayAddress_6 = false;
                        }
                    }
                   break;
                case TRANSFER_OUT_ADDRESS_6:
                   // Send the one character buffer.
                   if (DisplayAddress_6 & FirstDisplayByte)
                   {
                       AnalogValue = 0;
                       FirstDisplayByte = false;
                       AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0] << 8);
                   }
                   else if (DisplayAddress_6 & !FirstDisplayByte)
                   {
                       string WriteRcvChar = "";
                       FirstDisplayByte = true;

                       AnalogValue = AnalogValue | ((int)EPTReceiveData.cBlockBuf[0]);

                       if (ScaleFactor6 != 0)
                       {
                           FloatValue = AnalogValue * ScaleFactor6;
                           WriteRcvChar = String.Format("{0}", FloatValue);
                       }
                       else
                           WriteRcvChar = String.Format("{0}", AnalogValue);

                       DisplayAddress_1 = true;
                       DisplayAddress_2 = false;
                       DisplayAddress_3 = false;
                       DisplayAddress_4 = false;
                       DisplayAddress_5 = false;
                       DisplayAddress_6 = false;

                       Thread Display6Thread = new Thread(new ParameterizedThreadStart(DisplayValue6));
                    Display6Thread.Start(WriteRcvChar);
                    }
                    else if (!DisplayAddress_6 & FirstDisplayByte)
                    {
                        FirstPreviousCalcAddr = EPTReceiveData.Address;
                        FirstDisplayByte = false;
                        AnalogValue = 0;

                        DisplayAddress_1 = false;
                        DisplayAddress_2 = false;
                        DisplayAddress_3 = false;
                        DisplayAddress_4 = false;
                        DisplayAddress_5 = false;
                        DisplayAddress_6 = true;
                    }
                    else if (!DisplayAddress_6 & !FirstDisplayByte)
                    {
                        SecondPreviousCalcAddr = EPTReceiveData.Address + 1;
                        FirstDisplayByte = true;
                        AnalogValue = 0;
                        if ((FirstPreviousActualAddr == FirstPreviousCalcAddr) & (SecondPreviousActualAddr != SecondPreviousCalcAddr))
                        {
                            DisplayAddress_1 = true;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = false;
                        }
                        else
                        {
                            DisplayAddress_1 = false;
                            DisplayAddress_2 = false;
                            DisplayAddress_3 = false;
                            DisplayAddress_4 = false;
                            DisplayAddress_5 = false;
                            DisplayAddress_6 = true;
                        }
                    }
                   break;
                default:
                    break;
            }
        }

        public void DisplayValue1(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor1.Text = (string)WriteRcvChar; }));
        }

        public void DisplayValue2(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor2.Text = (string)WriteRcvChar; }));
        }

        public void DisplayValue3(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor3.Text = (string)WriteRcvChar; }));
        }

        public void DisplayValue4(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor4.Text = (string)WriteRcvChar; }));
        }

        public void DisplayValue5(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor5.Text = (string)WriteRcvChar; }));
        }

        public void DisplayValue6(object WriteRcvChar)
        {
            this.Invoke(new MethodInvoker(delegate() { tbMonitor6.Text = (string)WriteRcvChar; }));
        }

        private void btnResetBlock1_Click(object sender, EventArgs e)
        {
            tbMonitor1.Clear();
        }

        private void btnRstMonitor2_Click(object sender, EventArgs e)
        {
            tbMonitor2.Clear();
        }

        private void btnRstMonitor3_Click(object sender, EventArgs e)
        {
            tbMonitor3.Clear();
        }

        private void btnRstMonitor4_Click(object sender, EventArgs e)
        {
            tbMonitor4.Clear();
        }

        private void btnRstMonitor5_Click(object sender, EventArgs e)
        {
            tbMonitor5.Clear();
        }

        private void btnRstMonitor6_Click(object sender, EventArgs e)
        {
            tbMonitor6.Clear();
        }

        private void btnSetScaleFactor_Click(object sender, EventArgs e)
        {
            ScaleFactorMenuOpenWindow();
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte((char)2, (char)1);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte((char)2, (char)0);
        }
    }

    public class Transfer
    {
        public int Command;
        public int Address;
        public int Length;
        public int Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Transfer()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }
}
